/*
 ============================================================================
 Name		 : SocketsEngine.cpp
 Copyright   : Symbian Academy, 2007
 Description : The sockets engine is able to connect a socket and manages
               the reader and writer active objects.
============================================================================
*/

#include "SocketsEngine.h"

CSocketsEngine::CSocketsEngine(MSocketsNotifier* aNotifier) :
	CActive( EPriorityStandard), // Standard priority
			iNotifier(aNotifier)
	{
	}

CSocketsEngine* CSocketsEngine::NewLC(MSocketsNotifier* aNotifier)
	{
	CSocketsEngine* self = new ( ELeave ) CSocketsEngine(aNotifier);
	CleanupStack::PushL ( self);
	self->ConstructL ();
	return self;
	}

CSocketsEngine* CSocketsEngine::NewL(MSocketsNotifier* aNotifier)
	{
	CSocketsEngine* self = CSocketsEngine::NewLC (aNotifier);
	CleanupStack::Pop (); // self;
	return self;
	}

void CSocketsEngine::ConstructL()
	{
	CActiveScheduler::Add ( this); // Add to scheduler
	
	// Edit 5: Connect to the socket server
	//         Handle any errors that this method may return,
	//         e.g. by leaving.
	
	
	// Edit 6: Create new instances of the writer and reader
	//         active objects (iWriter, iReader).
	//         Pass a reference to the socket to these classes,
	//         as well as the iNotifier variable so that they
	//         can directly inform the listener about errors
	//         or received messages.

	
	iEngineState = EInitialized;
	}

CSocketsEngine::~CSocketsEngine()
	{
	// Edit 7: Delete the iWriter and iReader instance variables
	
	// Edit 8: Cancel any request that might still be outstanding
	//         (-> Active Object!)

	// Edit 9: Close the connection to the socket and the socket server
	
	
	}

void CSocketsEngine::DoCancel()
	{
	// Edit 10: If the engine is currently connecting (-> iEngineState!),
	//          cancel the connection process through the socket.
	//          While this method is only called when a request is active,
	//          the check of the state would become necessary when using
	//          this class for DNS lookup as well - in that case, a different
	//          cancel-method would have to be called.
	
	}

void CSocketsEngine::ConnectL()
	{
	// Edit 11: Check if the engine is not already in a connected state.
	//          (the custom iEngineState variable shouldn't indicate that
	//          the engine is already connected or currently connecting).
	
	
		// Edit 12: Make sure the AO is not currently active, e.g. by
		//          canceling any active request.
		
		// Edit 13: Open a (streaming) TCP socket
		//         Handle any errors that this method may return,
		//         e.g. by leaving.
		//         For the required parameters for the address family,
		//         socket type and protocol, see for example:
		//         SDK Help:  Symbian OS v9.2  Symbian OS guide  
		//         Networking  Using TCP/IP (INSOCK)  How to use TCP/IP  
		//         How to connect and disconnect
	
		// Edit 14: Connect the socket to the address provided by the argument.
	
		// Edit 15: Set the iEngineState variable to EConnecting.
	
		// Edit 16: Set the AO to active so that the active scheduler will know
		//          that a request is active.
	
		
	}

TBool CSocketsEngine::IsConnected()
	{
	return (iEngineState != EInitialized && iEngineState != EUninitialized);
	}

void CSocketsEngine::RunL()
	{
	switch (iEngineState)
		{
		case EConnecting:
			if ( iStatus == KErrNone)
				{
				// Edit 17: Set the iEngineState variable to EConnected and
				//          start a read request through the iReader,
				//          so that this class is waiting for incoming messages
				//          throgh the connected socket.
				
				
				_LIT(KConnected, "Connected");
				iNotifier->NotifyInfo (KErrNone, KConnected);
				}
			else
				{
				iEngineState = EInitialized;
				
				// Edit 18: Close the socket again to prepare it for another
				//          connection attempt.
				
				
				_LIT(KConnFailed, "Connection failed");
				iNotifier->NotifyInfo (iStatus.Int (), KConnFailed);
				}
			break;
		}
	}

void CSocketsEngine::IssueSend(const TDesC8& aMessage)
	{
	if ( iEngineState == EConnected)
		{
		iWriter->IssueSend (aMessage);
		}
	else
		{
		_LIT(KDisconnected, "Not connected");
		iNotifier->NotifyInfo (KErrDisconnected, KDisconnected);
		}
	}
